import xbmc
from helper import utils
from database import dbio
from emby import listitem
from core import common

Pictures = []
PlayerId = -1
PlayerPause = False
WatchTogetherSeekOffset = 0
WatchTogetherInProgress = False
WatchTogetherParticipants = []
WatchTogetherParticipantsAcks = []
# https://github.com/xbmc/xbmc/blob/master/xbmc/interfaces/json-rpc/schema/methods.json

def ClearPlaylist(PlaylistId):
    utils.SendJson(f'{{"jsonrpc": "2.0", "id": 1, "method": "Playlist.Clear", "params": {{"playlistid": {PlaylistId}}}}}')
    xbmc.log("Emby.helper.playerops: [ ClearPlaylist ]", 1) # LOGINFO

def InsertPlaylist(PlaylistId, Position, KodiType, KodiId):
    utils.SendJson(f'{{"jsonrpc": "2.0", "id": 1, "method": "Playlist.Insert", "params": {{"playlistid": {PlaylistId}, "position": {Position}, "item": {{"{KodiType}id": {KodiId}}}}}}}')
    xbmc.log("Emby.helper.playerops: [ InsertPlaylist ]", 1) # LOGINFO

def GetPlayerPosition(PlayerIdLocal):
    Result = utils.SendJson(f'{{"jsonrpc":"2.0", "method":"Player.GetProperties", "params":{{"playerid":{PlayerIdLocal},"properties": ["position"]}}, "id": 1}}').get("result", {})

    if Result:
        xbmc.log("Emby.helper.playerops: [ GetPlayerPosition ]", 1) # LOGINFO
        return Result.get("position", -1)

    xbmc.log(f"Emby.helper.playerops: GetPlayerPosition failed: Result={Result}", 2) # LOGWARNING
    return -1

def GetPlaylistSize(PlaylistId):
    Result = utils.SendJson(f'{{"jsonrpc":"2.0", "method":"Playlist.GetProperties", "params":{{"playlistid":{PlaylistId},"properties": ["size"]}}, "id": 1}}').get("result", {})

    if Result:
        xbmc.log("Emby.helper.playerops: [ GetPlaylistSize ]", 1) # LOGINFO
        return Result.get("size", 0)

    xbmc.log(f"Emby.helper.playerops: GetPlaylistSize failed: Result={Result}", 2) # LOGWARNING
    return 0

def PlayPlaylistItem(PlaylistId, Index):
    utils.SendJson(f'{{"jsonrpc":"2.0","method":"Player.Open","params":{{"item":{{"playlistid":{PlaylistId},"position":{Index}}}}},"id":1}}')

def GetFilenameandpath():
    if PlayerId != -1:
        Result = utils.SendJson('{"jsonrpc": "2.0", "method": "xbmc.GetInfoLabels", "params":{"labels": ["player.Filenameandpath"]}, "id": 1}').get("result", {})

        if Result:
            xbmc.log("Emby.helper.playerops: [ GetFilenameandpath ]", 1) # LOGINFO
            return Result.get("player.Filenameandpath", "")

    xbmc.log(f"Emby.helper.playerops: GetFilenameandpath failed: PlayerId={PlayerId} / Result={Result}", 2) # LOGWARNING
    return ""

def AddSubtitle(Path):
    if PlayerId != -1:
        utils.SendJson(f'{{"jsonrpc":"2.0", "method":"Player.AddSubtitle", "params":{{"playerid":{PlayerId}, "subtitle":"{Path}"}}, "id": 1}}')
    else:
        xbmc.log(f"Emby.helper.playerops: AddSubtitle failed: PlayerId={PlayerId}", 2) # LOGWARNING

def SetSubtitle(Enable):
    if PlayerId != -1:
        if Enable:
            utils.SendJson(f'{{"jsonrpc":"2.0", "method":"Player.SetSubtitle", "params":{{"playerid":{PlayerId}, "subtitle":"on"}}, "id": 1}}')
        else:
            utils.SendJson(f'{{"jsonrpc":"2.0", "method":"Player.SetSubtitle", "params":{{"playerid":{PlayerId}, "subtitle":"off"}}, "id": 1}}')

        xbmc.log(f"Emby.helper.playerops: [ SetSubtitle ] {Enable}", 1) # LOGINFO
    else:
        xbmc.log(f"Emby.helper.playerops: SetSubtitle failed: PlayerId={PlayerId}", 2) # LOGWARNING

def SetRepeatOff():
    if PlayerId != -1:
        utils.SendJson(f'{{"jsonrpc":"2.0", "method":"Player.SetRepeat", "params":{{"playerid":{PlayerId}, "repeat":"off"}}}}')
        xbmc.log("Emby.helper.playerops: [ SetRepeatOff ]", 1) # LOGINFO
    else:
        xbmc.log(f"Emby.helper.playerops: SetRepeatOff failed: PlayerId={PlayerId}", 2) # LOGWARNING

def SetRepeatOneTime():
    if PlayerId != -1:
        utils.SendJson(f'{{"jsonrpc":"2.0", "method":"Player.SetRepeat", "params":{{"playerid":{PlayerId}, "repeat":"one"}}}}')
        xbmc.log("Emby.helper.playerops: [ SetRepeatOneTime ]", 1) # LOGINFO
    else:
        xbmc.log(f"Emby.helper.playerops: SetRepeatOneTime failed: PlayerId={PlayerId}", 2) # LOGWARNING

def RemovePLaylistItem(PlaylistId, Index):
    utils.SendJson(f'{{"jsonrpc":"2.0", "method":"Playlist.Remove", "params":{{"playlistid":{PlaylistId}, "position":{Index}}}}}')

def Next():
    if PlayerId != -1:
        utils.SendJson(f'{{"jsonrpc":"2.0","method":"Player.GoTo","params":{{"playerid":{PlayerId},"to":"next"}},"id":1}}')
        xbmc.log("Emby.helper.playerops: [ Next ]", 1) # LOGINFO
    else:
        xbmc.log(f"Emby.helper.playerops: Next failed: PlayerId={PlayerId}", 2) # LOGWARNING

    globals()['PlayerPause'] = False

def Previous():
    if PlayerId != -1:
        utils.SendJson(f'{{"jsonrpc":"2.0","method":"Player.GoTo","params":{{"playerid":{PlayerId},"to":"previous"}},"id":1}}')
        xbmc.log("Emby.helper.playerops: [ Previous ]", 1) # LOGINFO
    else:
        xbmc.log(f"Emby.helper.playerops: Previous failed: PlayerId={PlayerId}", 2) # LOGWARNING

    globals()['PlayerPause'] = False

def Stop():
    if PlayerId != -1:
        utils.SendJson(f'{{"jsonrpc":"2.0","method":"Player.Stop","params":{{"playerid":{PlayerId}}},"id":1}}')
        xbmc.log("Emby.helper.playerops: [ Stop ]", 1) # LOGINFO
    else:
        xbmc.log(f"Emby.helper.playerops: Stop failed: PlayerId={PlayerId}", 2) # LOGWARNING

    globals()['PlayerPause'] = False

def PauseToggle():
    if PlayerId != -1:
        utils.SendJson(f'{{"jsonrpc":"2.0","method":"Player.PlayPause","params":{{"playerid":{PlayerId}}},"id":1}}')
        xbmc.log("Emby.helper.playerops: [ PauseToggle ]", 1) # LOGINFO
    else:
        xbmc.log(f"Emby.helper.playerops: PauseToggle failed: PlayerId={PlayerId}", 2) # LOGWARNING

def Pause():
    if PlayerId != -1 and not PlayerPause:
        utils.SendJson(f'{{"jsonrpc":"2.0","method":"Player.PlayPause","params":{{"playerid":{PlayerId}}},"id":1}}')
        globals()['PlayerPause'] = True
        xbmc.log("Emby.helper.playerops: [ Pause ]", 1) # LOGINFO
    else:
        xbmc.log(f"Emby.helper.playerops: Pause failed: PlayerId={PlayerId} / PlayerPause={PlayerPause}", 2) # LOGWARNING

def Unpause():
    if PlayerId != -1 and PlayerPause:
        utils.SendJson(f'{{"jsonrpc":"2.0","method":"Player.PlayPause","params":{{"playerid":{PlayerId}}},"id":1}}')
        globals()['PlayerPause'] = False
        xbmc.log("Emby.helper.playerops: [ Unpause ]", 1) # LOGINFO
    else:
        xbmc.log(f"Emby.helper.playerops: Unpause failed: PlayerId={PlayerId} / PlayerPause={PlayerPause}", 2) # LOGWARNING

def Seek(SeekPositionTicks, EmbyFormat=True):
    if PlayerId != -1:
        if EmbyFormat:
            if WatchTogetherSeekOffset:
                SeekPositionTicks += (utils.unixtimeInMicroseconds() - WatchTogetherSeekOffset) * 100
                globals()['WatchTogetherSeekOffset'] = 0.0

            seektime = int(SeekPositionTicks / 10000.0)
        else:
            seektime = int(SeekPositionTicks)

        milliseconds = int(seektime % 1000)
        seconds = int((seektime / 1000) % 60)
        minutes = int((seektime / 60000) % 60)
        hours =  int((seektime / 3600000) % 24)
        utils.SendJson(f'{{"jsonrpc":"2.0","method":"Player.Seek","params":{{"playerid":{PlayerId},"value":{{"time":{{"hours":{hours},"minutes":{minutes},"seconds":{seconds},"milliseconds": {milliseconds}}}}}}},"id":1}}')
        xbmc.log(f"Emby.helper.playerops: [ seek / {seektime} ]", 1) # LOGINFO

def PlayBackPosition():
    if PlayerId != -1:
        Result = utils.SendJson(f'{{"jsonrpc":"2.0","method":"Player.GetProperties","params":{{"playerid":{PlayerId},"properties": ["time"]}},"id":1}}').get("result", {})

        if Result:
            TimeStamp = Result.get("time", {})

            if TimeStamp:
                return (TimeStamp['hours'] * 3600000 + TimeStamp['minutes'] * 60000 + TimeStamp['seconds'] * 1000 + TimeStamp['milliseconds']) * 10000

        xbmc.log(f"Emby.helper.playerops: PlayBackPosition failed: Result={Result}", 2) # LOGWARNING
    else:
        xbmc.log(f"Emby.helper.playerops: PlayBackPosition failed: PlayerId={PlayerId}", 2) # LOGWARNING

    return -1

def PlayBackDuration():
    if PlayerId != -1:
        Result = utils.SendJson(f'{{"jsonrpc":"2.0","method":"Player.GetProperties","params":{{"playerid":{PlayerId},"properties": ["totaltime"]}},"id":1}}').get("result", {})

        if Result:
            TimeStamp = Result.get("totaltime", {})

            if TimeStamp:
                return (TimeStamp['hours'] * 3600000 + TimeStamp['minutes'] * 60000 + TimeStamp['seconds'] * 1000 + TimeStamp['milliseconds']) * 10000

        xbmc.log(f"Emby.helper.playerops: PlayBackDuration failed: Result={Result}", 2) # LOGWARNING
    else:
        xbmc.log(f"Emby.helper.playerops: PlayBackDuration failed: PlayerId={PlayerId}", 2) # LOGWARNING

    return 0

def init_WatchTogether(EmbyId, EmbyServer):
    globals()['WatchTogetherInProgress'] = True
    PlayEmby((EmbyId,), "PlayNow", 0, -1, EmbyServer)

    for _ in range(30): # Timout 30 seconds
        if PlayerId != -1:
            Pause()
            Seek(0, False)
            return

        if not WatchTogetherInProgress or utils.sleep(1): # wait here for user confirmation or timeout
            return

def terminate_WatchTogether():
    globals()['WatchTogetherInProgress'] = False

    if PlayerId != -1:
        Stop()

def PlayEmby(ItemIds, PlayCommand, StartIndex, StartPositionTicks, EmbyServer):
    ItemsData = []
    EmbyIds = ()
    Reference = {}
    PlayItem = ()
    embydb = dbio.DBOpenRO(EmbyServer.ServerData['ServerId'], "AddPlaylistItem")

    for Index, EmbyID in enumerate(ItemIds):
        KodiId, KodiType = embydb.get_KodiId_KodiType_by_EmbyId_EmbyLibraryId(EmbyID)
        EmbyIDStr = str(EmbyID)

        if KodiId: # synced content
            ItemsData.append((True, KodiType, KodiId))
        else: # not synced content
            ItemsData.append(())
            Reference[EmbyIDStr] = Index
            EmbyIds += (EmbyIDStr,)

    dbio.DBCloseRO(EmbyServer.ServerData['ServerId'], "AddPlaylistItem")

    # query unsynced items
    if EmbyIds:
        Items = EmbyServer.API.get_Item(",".join(EmbyIds), ['Everything'], True, False, False)

        for Item in Items:
            li = listitem.set_ListItem(Item, EmbyServer.ServerData['ServerId'])
            path, Type = common.get_path_type_from_item(EmbyServer.ServerData['ServerId'], Item)
            ItemsData[Reference[Item['Id']]] = (False, Type, li, path)

    # Add items to playlist
    PlaylistPos = (GetPlayerPosition(0), GetPlayerPosition(1))
    PlaylistSize = (GetPlaylistSize(0), GetPlaylistSize(1))
    Counter = [0, 0, 0]

    if PlayCommand in ("PlayNow", "PlayNext"):
        Offset = (PlaylistPos[0] + 1, PlaylistPos[1] + 1, 0)
    else:
        Offset = (PlaylistSize[0], PlaylistSize[1], 0)

    for Index, ItemData in enumerate(ItemsData):
        if ItemData[1] in ("song", "a"):
            PlayerIdPlaylistId = 0
        elif ItemData[1] == "p":
            PlayerIdPlaylistId = 2
        else:
            PlayerIdPlaylistId = 1

        InsertPosition = Offset[PlayerIdPlaylistId] + Counter[PlayerIdPlaylistId]

        if PlayerIdPlaylistId != 2:
            if ItemData[0]: # synced item
                InsertPlaylist(PlayerIdPlaylistId, InsertPosition, ItemData[1], ItemData[2])
            else:
                utils.Playlists[PlayerIdPlaylistId].add(ItemData[3], ItemData[2], index=InsertPosition)
        else:
            Pictures.append((path, li))

        if Index == StartIndex:
            PlayItem = (PlayerIdPlaylistId, InsertPosition)

        Counter[PlayerIdPlaylistId] += 1

    if PlayItem:
        if PlayItem[0] == 2: # picture
            globals()["Pictures"][PlayItem[1]][1].select(True)
            xbmc.executebuiltin('Action(Stop)')
            xbmc.executebuiltin('Action(Back)')
            ClearPlaylist(2)
            xbmc.executebuiltin("ReplaceWindow(10002,'plugin://{utils.PluginId}/?mode=remotepictures&position={PlayItem[1]}')")
        else:
            PlayPlaylistItem(PlayItem[0], PlayItem[1])

            if StartPositionTicks != -1:
                Seek(StartPositionTicks, True)
